package com.famoco.barcodesampleapp.repository

import android.content.Context

import android.os.Build
import android.view.KeyEvent
import com.famoco.barcodesampleapp.repository.scanner.ScannerManager
import com.famoco.barcodesampleapp.repository.scanner.ScannerManager320
import com.famoco.barcodesampleapp.repository.scanner.ScannerManager325
import dagger.hilt.android.qualifiers.ApplicationContext
import javax.inject.Inject
import javax.inject.Singleton

@Singleton
class MainRepository @Inject constructor(@ApplicationContext private val context: Context) {

    companion object {
        private val TAG = MainRepository::class.simpleName
        private const val FX325_BUILD_MODEL = "FX325,1"
        private const val FX335_BUILD_MODEL = "FX335,1"
    }

    private val scannerManager = buildScannerManager()

    val lastScanBarcodeResult = scannerManager.lastScanBarcodeResult
    val scanning = scannerManager.scanning
    val enableFillEditText = scannerManager.enableFillEditText
    val continuousScanPhysicalButton = scannerManager.continuousScanPhysicalButton
    val nonStopScan = scannerManager.nonStopScan

    private fun isFX3X5(): Boolean {
        return (Build.MODEL == FX325_BUILD_MODEL) || (Build.MODEL == FX335_BUILD_MODEL)
    }

    /**
     * Creates scanner manager instance according to the current device
     */
    private fun buildScannerManager(): ScannerManager {
        return if (isFX3X5()) {
            ScannerManager325(context)
        } else {
            ScannerManager320(context)
        }
    }

    suspend fun enableScanner() = scannerManager.enableScanner()
    suspend fun disableScanner() = scannerManager.disableScanner()
    suspend fun startScan() = scannerManager.startScan()
    suspend fun stopScan() = scannerManager.stopScan()
    suspend fun clearBarcodeData() = scannerManager.clearBarcodeData()
    suspend fun enableFillEditText(enable: Boolean) = scannerManager.enableFillEditText(enable)
    suspend fun setContinuousScanPhysicalButton(enable: Boolean) = scannerManager.setContinuousScanPhysicalButton(enable)
    suspend fun setNonStopScan(enable: Boolean) = scannerManager.setNonStopScan(enable)
    suspend fun handlePhysicalButtonPress(keyCode: Int, event: KeyEvent?) = scannerManager.handlePhysicalButtonPress(keyCode, event)
    suspend fun handlePhysicalButtonRelease(keyCode: Int, event: KeyEvent?) = scannerManager.handlePhysicalButtonRelease(keyCode, event)
}