package com.famoco.barcodesampleapp.repository.scanner

import android.app.Instrumentation
import android.content.Context
import android.os.Build
import android.view.KeyCharacterMap
import android.view.KeyEvent
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import kotlinx.coroutines.*
import java.util.concurrent.Executors
import kotlin.coroutines.CoroutineContext

abstract class ScannerManager(protected val context: Context) {
    protected val coroutineScope = CustomCoroutineScope()

    protected val _lastScanBarcodeResult = MutableLiveData<String?>(null)
    val lastScanBarcodeResult: LiveData<String?> = _lastScanBarcodeResult

    protected val _scanning = MutableLiveData(false)
    val scanning: LiveData<Boolean> = _scanning

    protected val _enableFillEditText = MutableLiveData(false)
    val enableFillEditText: LiveData<Boolean> = _enableFillEditText

    protected val _continuousScanPhysicalButton = MutableLiveData(false)
    val continuousScanPhysicalButton: LiveData<Boolean> = _continuousScanPhysicalButton

    protected val _nonStopScan = MutableLiveData(false)
    val nonStopScan: LiveData<Boolean> = _nonStopScan

    abstract suspend fun enableScanner()
    abstract suspend fun disableScanner()
    abstract suspend fun startScan()
    abstract suspend fun stopScan()
    abstract suspend fun clearBarcodeData()
    abstract suspend fun enableFillEditText(enable: Boolean)
    abstract suspend fun setContinuousScanPhysicalButton(enable: Boolean)
    abstract suspend fun setNonStopScan(enable: Boolean)
    abstract suspend fun handlePhysicalButtonPress(keyCode: Int, event: KeyEvent?)
    abstract suspend fun handlePhysicalButtonRelease(keyCode: Int, event: KeyEvent?)

    /**
     * Send scanned digit as keyboard entry if option is enabled
     */
    protected suspend fun sendKeyEvents(scanResult: String) {
        val chars = scanResult.toCharArray()
        val charMap: KeyCharacterMap = if (Build.VERSION.SDK_INT >= 11)
            KeyCharacterMap.load(KeyCharacterMap.VIRTUAL_KEYBOARD) else KeyCharacterMap.load(
            KeyCharacterMap.ALPHA
        )

        val events: Array<KeyEvent> = charMap.getEvents(chars)

        val inst = Instrumentation()
        events.filterIndexed { index, keyEvent -> index % 2 == 0 }.forEach {
            inst.sendKeyDownUpSync(it.keyCode)
        }
    }

    class CustomCoroutineScope internal constructor() : CoroutineScope {
        private val dispatcher = Executors.newSingleThreadExecutor()
            .asCoroutineDispatcher()

        override val coroutineContext: CoroutineContext =
            dispatcher + Job() + CoroutineExceptionHandler { coroutineContext: CoroutineContext, throwable: Throwable ->
                GlobalScope.launch { println("Caught $throwable") }
            }
    }
}