package com.famoco.barcodesampleapp.ui.main

import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import com.famoco.barcodesampleapp.R
import com.famoco.barcodesampleapp.databinding.FragmentMainBinding
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class MainFragment : Fragment() {

    companion object {
        private val TAG = MainFragment::class.simpleName
    }

    private var _binding: FragmentMainBinding? = null
    private val binding get() = _binding!!

    private val viewModel: MainViewModel by viewModels()

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentMainBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.lastScanBarcodeResult.observe(viewLifecycleOwner) {
            if (it == null) {
                return@observe
            }
            Log.i(TAG, "barcode received: $it")
            binding.barcodeTextView.text = it
        }

        viewModel.scanning.observe(viewLifecycleOwner) {
            Log.i(TAG, "scanning: $it")
            if (!it) {
                binding.scanButton.text = getString(R.string.scan)
                binding.scanButton.setOnClickListener {
                    viewModel.startBarcodeScan()
                }
            } else {
                binding.scanButton.text = getString(R.string.stop)
                binding.scanButton.setOnClickListener {
                    viewModel.stopBarcodeScan()
                }
            }
        }

        viewModel.enableFillEditText.observe(viewLifecycleOwner) {
            binding.enableFillEditTextCheckbox.isChecked = it
        }

        viewModel.continuousScanPhysicalButton.observe(viewLifecycleOwner) {
            binding.continuousScanCheckbox.isChecked = it
        }

        viewModel.nonStopScan.observe(viewLifecycleOwner) {
            binding.nonStopScanCheckbox.isChecked = it
        }

        binding.clearButton.setOnClickListener {
            viewModel.clearBarcodeData()
            binding.sampleEditText.text = null
        }

        binding.enableFillEditTextCheckbox.setOnCheckedChangeListener { buttonView, isChecked ->
            viewModel.enableFillEditText(isChecked)
        }

        binding.continuousScanCheckbox.setOnCheckedChangeListener { buttonView, isChecked ->
            viewModel.setContinuousScanPhysicalButton(isChecked)
        }

        binding.nonStopScanCheckbox.setOnCheckedChangeListener { buttonView, isChecked ->
            viewModel.setNonStopScan(isChecked)
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

}