package com.famoco.barcodesampleapp.ui.main

import android.view.KeyEvent
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.famoco.barcodesampleapp.repository.MainRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class MainViewModel @Inject constructor(
    private val repository: MainRepository
) : ViewModel() {
    val lastScanBarcodeResult = repository.lastScanBarcodeResult
    val scanning = repository.scanning
    val enableFillEditText = repository.enableFillEditText
    val continuousScanPhysicalButton = repository.continuousScanPhysicalButton
    val nonStopScan = repository.nonStopScan

    fun clearBarcodeData() {
        viewModelScope.launch(Dispatchers.IO) {
            repository.clearBarcodeData()
        }
    }

    fun enableScanner() {
        viewModelScope.launch(Dispatchers.IO) {
            repository.enableScanner()
        }
    }

    fun disableScanner() {
        viewModelScope.launch(Dispatchers.IO) {
            repository.disableScanner()
        }
    }

    fun startBarcodeScan() {
        viewModelScope.launch(Dispatchers.IO) {
            repository.startScan()
        }
    }

    fun stopBarcodeScan() {
        viewModelScope.launch(Dispatchers.IO) {
            repository.stopScan()
        }
    }

    fun enableFillEditText(enable: Boolean) {
        viewModelScope.launch(Dispatchers.IO) {
            repository.enableFillEditText(enable)
        }
    }

    fun setContinuousScanPhysicalButton(enable: Boolean) {
        viewModelScope.launch(Dispatchers.IO) {
            repository.setContinuousScanPhysicalButton(enable)
        }
    }

    fun setNonStopScan(enable: Boolean) {
        viewModelScope.launch(Dispatchers.IO) {
            repository.setNonStopScan(enable)
        }
    }

    fun handlePhysicalButtonPress(keyCode: Int, event: KeyEvent?) {
        viewModelScope.launch(Dispatchers.IO) {
            repository.handlePhysicalButtonPress(keyCode, event)
        }
    }

    fun handlePhysicalButtonRelease(keyCode: Int, event: KeyEvent?) {
        viewModelScope.launch(Dispatchers.IO) {
            repository.handlePhysicalButtonRelease(keyCode, event)
        }
    }
}